@@ -21,8 +21,8 @@ def entry():
 
 
 @click.command(name="mint")
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def mint(keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -43,9 +43,9 @@ async def config():
     pass
 
 @click.command(name = "set")
-@click.option("--program_id", "-p")
-@click.option("--url", "-u")
-@click.option("--keypair", "-k")
+@click.option("--program_id", "-p", help = "Enter the program Id of the validator instance transactions will default to")
+@click.option("--url", "-u", help = "Enter the network you want to connect to. Options: mainnet, testnet, devnet, or a custom url.")
+@click.option("--keypair", "-k", help="Enter the path to the keypair that transactions will be signed with by default.")
 def set(program_id, url, keypair):
     assert program_id or url or keypair, "No options specified. Use --help for more information."
     if program_id:
@@ -87,8 +87,8 @@ config.add_command(set)
 config.add_command(get)
 
 @click.command(name="init_rebalance")
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def initialize_rebalancing(keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -103,8 +103,8 @@ async def initialize_rebalancing(keypair, log_level):
     await client.close()
 
 @click.command(name="finalize_rebalance")
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def finalize_rebalancing(keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -119,9 +119,9 @@ async def finalize_rebalancing(keypair, log_level):
     await client.close()
 
 @click.command(name="init")
-@click.option('--validator', '-v', default = get_keypair_path())
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.option('--validator', '-v', default = get_keypair_path(), help = "Enter the path to the validator id, or the public key of the validator id for this instance")
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def ingl(keypair, validator, log_level):
 
     init_commission = 105
@@ -195,15 +195,25 @@ async def ingl(keypair, validator, log_level):
         counter += 1
         governance_expiration_time = click.prompt("How long should a governance take to expire? (in days) (>=35 days, <=365 days) ", type=int)
     governance_expiration_time = governance_expiration_time * 86_400
-    rarities = [100, 500, 1500, 2900, 5000]#TODO: Make this dynamic
-    rarity_name = ['Mythic', 'Exalted', 'Rare', 'Uncommon', 'Common']#TODO: Make this dynamic
     twitter_handle = click.prompt("Enter the Twitter handle of the validator: ", type=str)
     discord_invite = click.prompt("Enter the Discord Invite of the validator: ", type=str)
     validator_name = click.prompt("Enter the Name of the validator: ", type=str)
-    collection_uri = click.prompt("Enter the Collection URI of the validator: ", type=str)
     website = click.prompt("Enter the Website of the validator: ", type=str)
     default_uri = click.prompt("Enter the Default URI of the validator: ", type=str)
 
+    json_data = {}
+    while 'uris' not in json_data:
+        json_path = click.prompt("Enter the Collection path to the config json: ", type=str)
+        try:
+            f = open(f"{json_path}", "r")
+            json_data = json.load(f)
+            f.close()
+        except Exception as e:
+            print("Invalid Json Path. ")
+            return
+    collection_uri = json_data['collection_uri']
+    rarity_names = json_data['rarity_names']
+    rarities = json_data['rarities']
 
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -219,14 +229,14 @@ async def ingl(keypair, validator, log_level):
         print("Invalid Validator Input. ")
         return
         
-    t_dets = await ingl_init(payer_keypair, validator_key, init_commission, max_primary_stake, nft_holders_share, initial_redemption_fee, is_validator_switchable, unit_backing, redemption_fee_duration, proposal_quorum, creator_royalty, governance_expiration_time, rarities, rarity_name, twitter_handle, discord_invite, validator_name, collection_uri, website, default_uri, client, log_level,)
+    t_dets = await ingl_init(payer_keypair, validator_key, init_commission, max_primary_stake, nft_holders_share, initial_redemption_fee, is_validator_switchable, unit_backing, redemption_fee_duration, proposal_quorum, creator_royalty, governance_expiration_time, rarities, rarity_names, twitter_handle, discord_invite, validator_name, collection_uri, website, default_uri, client, log_level,)
     print(t_dets)
     await client.close()
 
 
 @click.command(name="process_rewards")
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def process_vote_account_rewards(keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -241,8 +251,8 @@ async def process_vote_account_rewards(keypair, log_level):
     await client.close()
 
 @click.command(name='create_vote_account')
-@click.option('--val_keypair', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.option('--val_keypair', default = get_keypair_path(), help="Enter the path to the validator id keypair json file. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def process_create_vote_account(val_keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -257,9 +267,9 @@ async def process_create_vote_account(val_keypair, log_level):
     await client.close()
 
 @click.command(name='delegate')
-@click.argument('mint_id')
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.argument('mint_id', type=str, help="Enter the mint_id of the NFT you want to delegate")
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def process_delegate_gem(keypair, mint_id, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -279,9 +289,9 @@ async def process_delegate_gem(keypair, mint_id, log_level):
     await client.close()
 
 @click.command(name = 'undelegate')
-@click.argument('mint_id')
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.argument('mint_id', type=str, help = 'mint_id of the NFT you want to undelegate')
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def process_undelegate_gem(keypair, mint_id, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -302,9 +312,9 @@ async def process_undelegate_gem(keypair, mint_id, log_level):
     await client.close()
 
 @click.command(name='init_governance')
-@click.argument('mint_id')
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.argument('mint_id', type=str, help="The mint_id of the NFT you want to initiate a governance with")
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def process_create_governance(keypair, mint_id, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -347,11 +357,11 @@ async def process_create_governance(keypair, mint_id, log_level):
     await client.close()
 
 @click.command(name='vote_governance')
-@click.argument('mint')
-@click.argument('numeration', type=int)
-@click.option('--vote', '-v', default='D')
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.argument('mint', type=str, help="Enter the mint address of the NFT you want to vote with.")
+@click.argument('numeration', type=int, help="Enter the numeration of the governance you want to vote on.")
+@click.option('--vote', '-v', default='D', help="Enter the vote you want to cast. D: For 'Dissapprove', A: For 'Approve', ")
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def process_vote_governance(keypair, mint, numeration, vote, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -373,9 +383,9 @@ async def process_vote_governance(keypair, mint, numeration, vote, log_level):
     await client.close()
 
 @click.command(name='finalize_governance')
-@click.argument('numeration', type=int)
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.argument('numeration', type=int, help="Enter the numeration of the governance you want to finalize")
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def process_finalize_governance(keypair, numeration, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -391,9 +401,9 @@ async def process_finalize_governance(keypair, numeration, log_level):
     await client.close()    
 
 @click.command(name='execute_governance')
-@click.argument('numeration', type=int)
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.argument('numeration', type=int, help="Enter the numeration of the governance you want to execute")
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def process_execute_governance(keypair, numeration, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -410,8 +420,8 @@ async def process_execute_governance(keypair, numeration, log_level):
 
 @click.command(name='upload_uris')
 @click.argument('json_path')
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def process_upload_uris(keypair, json_path, log_level):
     client_state = uasyncclient.is_connected()
     client = AsyncClient(rpc_url.target_network)
@@ -444,8 +454,8 @@ async def process_upload_uris(keypair, json_path, log_level):
     await client.close()
 
 @click.command(name='reset_uris')
-@click.option('--keypair', '-k', default = get_keypair_path())
-@click.option('--log_level', '-l', default = 2, type=int)
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
+@click.option('--log_level', '-l', default = 2, type=int, help="Precise Log_level you want the transaction to be logged at, and above(0 -> 5). 0: All logs,  ... 5: Only Errors")
 async def process_reset_uris(keypair, log_level):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -460,7 +470,7 @@ async def process_reset_uris(keypair, log_level):
     await client.close()
 
 @click.command(name='init_registry')
-@click.option('--keypair', '-k', default = get_keypair_path())
+@click.option('--keypair', '-k', default = get_keypair_path(), help="Enter the path to the keypair that will be used to sign this transaction. Defaults to the set config keypair")
 async def process_initialize_registry(keypair):
     client = AsyncClient(rpc_url.target_network)
     client_state = await client.is_connected()
@@ -474,8 +484,19 @@ async def process_initialize_registry(keypair):
     print(t_dets)
     await client.close()
 
-
-
+@click.command(name="get_vote_pubkey")
+@click.option('--program_id', '-p', default = get_program_id(), help="Enter the program_id of the validator instance you want to get the vote account pubkey for. Defaults to the set config program_id")
+async def process_get_vote_key(program_id):
+    try:
+        program_pubkey = parse_pubkey_input(program_id)
+    except Exception as e:
+        print("Invalid Pubkey Input, ", e)
+        return
+    print("Program_id: ", program_pubkey.pubkey)
+    expected_vote_pubkey, expected_vote_bump = Pubkey.find_program_address([bytes(ingl_constants.VOTE_ACCOUNT_KEY, "UTF-8")], program_pubkey.pubkey)
+    print("Vote Account Key: ", expected_vote_pubkey)
+    print("Vote Account Bump: ", expected_vote_bump)
+    return
 
 entry.add_command(mint)
 entry.add_command(initialize_rebalancing)
@@ -493,5 +514,6 @@ entry.add_command(config)
 entry.add_command(process_upload_uris)
 entry.add_command(process_reset_uris)
 entry.add_command(process_initialize_registry)
+entry.add_command(process_get_vote_key)
 if __name__ == '__main__':
     entry()
\ No newline at end of file
